package com.bitmechanic.maxq;

import org.apache.commons.cli.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.python.core.Py;
import org.python.core.PyException;
import org.python.util.PythonInterpreter;

//import java.io.BufferedReader;
import java.io.File;
//import java.io.InputStreamReader;

/**
 * todo Make default package configurable
 * todo Make default save location configurable
 * todo Remeber package and save location
 */
public class Main
{
  private static Config config = Config.getConfig();
  private static final String VERSION = "1.0.2";
  private static final Log log = LogFactory.getLog(Main.class);

  //private BufferedReader in;

  static public class CommandLineResults
  {
    public boolean noGui;
    public String[] tests;
  }

  private ProxyServer proxy;

  public ProxyServer getProxy()
  {
    return proxy;
  }

  //    private ArrayList actionList = new ArrayList();

  // This really should be a private method, but I want to be able
  // to test this using Junit, so make it public.  :-(
  public static void parseLine(String[] argv, CommandLineResults results)
  {
    Options options = buildOptions();
    try {
      CommandLineParser parser = new PosixParser();
      CommandLine line = parser.parse(options, argv);
      results.tests = new String[0];

      if (line.hasOption("h")) {
        printOptionsAndExit(options, 0);
      }

      // Report conflicting options.
      if (line.hasOption('q') && line.hasOption('d')) {
        System.err.println("MaxQ cannot both be verbose (-d) and quiet (-q) at the same time.");
        printOptionsAndExit(options, 2);
      }
      if ((line.hasOption('e') || line.hasOption('i'))
          && line.hasOption('r')) {
        System.err.println("The -i and -e options do not make when running an existing script (-r).");
        printOptionsAndExit(options, 2);
      }

      // Set the python.path
      if (line.hasOption("a")) {
        config.addPythonPath(line.getOptionValue("a"));
      }

      if (line.hasOption("d")) {
        config.setDebug(true);
      }

      if (line.hasOption("q")) {
        config.setQuiet(true);
      }

      if (line.hasOption("u")) {
        String[] urls = line.getOptionValues("u");
        if (urls.length < 2) {
          System.err.println("Missing required two arguments to --urlreplace switch");
          printOptionsAndExit(options, 2);
        }

        config.setExist(urls[0]);
        config.setReplace(urls[1]);
        HttpTestCase.setURLReplace(config.getExist(), config.getReplace());
      }

      if (line.hasOption("r")) {
        results.noGui = true;
      }

      if (line.hasOption("p")) {
        config.setPort(Integer.parseInt(line.getOptionValue("p")));
      }

      if (line.hasOption("i")) {
        config.setIncludePatterns(line.getOptionValues("i"));
      }

      if (line.hasOption("e")) {
        config.setExcludePatterns(line.getOptionValues("e"));
      }

      if (line.hasOption("g")) {
        config.setScriptArg(line.getOptionValue("g"));
      }

      // Any leftovers should be a file to open.
      results.tests = line.getArgs();
      if (results.tests.length > 1 && !results.noGui) {
        System.err.println("Only one file may be specified unless you run without the GUI (-r).");
        printOptionsAndExit(options, 2);
      }
    } catch (ParseException e) {
      System.err.println(e.getMessage());
      printOptionsAndExit(options, 2);
    }
  }

  static void printOptionsAndExit(Options options, int rc)
  {
    // automatically generate the help statement
    HelpFormatter formatter = new HelpFormatter();
    formatter.printHelp("usage: maxq [-p port] [-a python_include_path] [-q] [-d] [-u url1 url2] [-r] [test1.py ...]",
        options);
    System.exit(rc);
  }

  public static void main(String[] argv)
  {
    try {
      CommandLineResults cmdLine = new CommandLineResults();
      parseLine(argv, cmdLine);
      config.completeInit();

      Main m = new Main();

      if (cmdLine.noGui) {
        m.runTests(cmdLine.tests);
      } else {
        String test;
        if (cmdLine.tests.length > 0)
          test = cmdLine.tests[0];
        else
          test = null;
        m.startGUI(config.getPort(), test);
      }
    } catch (Exception e) {
      if (e.getClass() != Utils.UserException.class)
        e.printStackTrace();
      else
        log.error(e.getMessage());
      // We cannot just drop out of main() because swing will keep
      // running its thread.
      System.exit(1);
    }

    // We do not call System.exit(0) here because Swing will still be going.
  }

  private static Options buildOptions()
  {
    Options cmdOptions = new Options();
    Option opt;

    opt = new Option("d", "debug", false, "run in debug mode");
    cmdOptions.addOption(opt);
    opt = new Option("q", "quiet", false, "Quiet mode, only output on failure");
    cmdOptions.addOption(opt);

    cmdOptions.addOption(new Option("r", "run", false, "run test(s)"));

    opt = new Option("u", "urlreplace", true, "Replace all " +
            "instances of url1 with url2");
    opt.setArgs(2);
    cmdOptions.addOption(opt);

    cmdOptions.addOption("h", "help", false, "generate this page");
    cmdOptions.addOption("a", "path", true, "The PYTHONPATH for the " +
            "interpreter");
    cmdOptions.addOption("p", "port", true, "Port on which to run the " +
            "proxy, default is 8090");
    cmdOptions.addOption("g", "script-arg", true, "Argument to be made available to the script");

    opt = new Option("i", "include-get-patterns", true, "add include filter patterns to the recorder");
    opt.setArgs(Option.UNLIMITED_VALUES);
    cmdOptions.addOption(opt);

    opt = new Option("e", "exclude-get-patterns", true, "add exclude filter patterns to the recorder");
    opt.setArgs(Option.UNLIMITED_VALUES);
    cmdOptions.addOption(opt);

    return cmdOptions;
  }

  /**
   * Runs tests on command line.
   */
  public void runTests(String[] tests)
  {
    int i;
    for (i = 0; i < tests.length; i++) {
      String fileName = tests[i];
      log.info("Running: " + fileName);
      PythonInterpreter python = new PythonInterpreter();
      try {
        python.execfile(fileName);
      } catch (PyException e) {
        //e.printStackTrace();
        Py.printException(e);  // Does not output Java stack.
        System.exit(1);
      }
      log.info("OK");
    }

    System.exit(0);
  }

  public void startGUI(int port, String openPath) throws Exception
  {
    log.info("MaxQ Proxy Recorder v" + VERSION);

    // Why was this here?
    //in = new BufferedReader(new InputStreamReader(System.in));

    //getPort();
    proxy = new ProxyServer(port);
    proxy.start();

    //
    // Initialize GUI
    SwingMain gui = new SwingMain(proxy);
    if (openPath != null)
      gui.openFile(new File(openPath));
  }
}
