package com.bitmechanic.maxq;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Enumeration;
import java.util.Vector;

/**
 * A proxy server.  Creates a RequestHandler in a separate thread to handle
 * each connection it receives.  RequestHandlers call back here with the
 * data for the requests and response being handled, and we pass this on to
 * any observers that register themselves.
 */
public class ProxyServer extends Thread
{
  public static interface Observer
  {
    public void processRequest(HttpRequestHeader header, byte[] requestBody) throws Exception;

    public void processResponse(HttpRequestHeader header, byte[] response) throws Exception;
  }

  private Vector listeners;
  private ServerSocket srvSock;
  private int count;
  private static final Log log = LogFactory.getLog(ProxyServer.class);

  public ProxyServer() throws IOException
  {
    this(0);
  }

  /**
   * Attempts to start a proxy server on the specified port.
   * <p/>
   * if successful,it starts the proxy server on a new thread.
   */
  public ProxyServer(int port) throws IOException
  {
    srvSock = new ServerSocket(port);
    listeners = new Vector();
  }

  public void addObserver(Observer proxy)
  {
    listeners.addElement(proxy);
  }

  public void removeObserver(Observer proxy)
  {
    listeners.removeElement(proxy);
  }

  public void processRequest(HttpRequestHeader header, byte[] requestBody) throws Exception
  {
    for (Enumeration e = listeners.elements(); e.hasMoreElements();) {
      Observer pl = (Observer) e.nextElement();
      pl.processRequest(header, requestBody);
    }
  }

  public void processResponse(HttpRequestHeader header, byte[] response) throws Exception
  {
    for (Enumeration e = listeners.elements(); e.hasMoreElements();) {
      Observer pl = (Observer) e.nextElement();
      pl.processResponse(header, response);
    }
  }

  public void run()
  {
    log.info("Proxy server started on port: " + getLocalPort());
    while (true) {
      try {
        Socket s = srvSock.accept();
        RequestHandler handler = new RequestHandler(this, s);
        log.trace("New connection.  Creating new RequestHandler thread.");
        Thread t = new Thread(handler, "MaxQRequestHandler #" + count);
        t.start();
        count++;
      } catch (Throwable t) {
        SwingMain.reportOutsideException(t);
      }
    }
  }

  int getLocalPort()
  {
    return srvSock.getLocalPort();
  }
}

