package com.bitmechanic.maxq.generator;

import com.bitmechanic.maxq.Config;
import com.bitmechanic.maxq.HeaderValue;
import com.bitmechanic.maxq.IScriptAdapter;
import com.bitmechanic.maxq.Param;
import com.bitmechanic.maxq.Utils.UserException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;

/**
 * Generate a java JUnit test case.
 * Take a look at the CodeGenerator Interface to see how to specify which one you want.
 * The important information is that this class generates Java code.
 *
 * @author Robert Dawson robert@rojotek.com
 * @author Wayne Hearn wayne_hearn@dell.com
 * @see IScriptGenerator
 */
public class JavaCodeGenerator extends AbstractCodeGenerator {

	private Config config = Config.getConfig();
	private String baseClasspath;
	private static final Log log = LogFactory.getLog(JavaCodeGenerator.class);

	public JavaCodeGenerator(IScriptAdapter adapter) {
		super(adapter, new String[] {"public class (\\w+) extends", "public (\\w+)\\(String name\\)", "public void test(\\w+)\\(\\) throws"});
		File basePath = new File(config.getProperty("generator.java.sourcepath", ""));
		this.baseClasspath = basePath.getAbsolutePath();
		log.debug(this.baseClasspath);
	}

	public void setStruts(boolean struts) {
	}

	public String[] getValidFileExtensions() {
		return new String[] {".java"};
	}

	public void doNew() {
		StringBuffer result = new StringBuffer();
		result.append(getPackageDeclaration());
		result.append("// This class was generated by MaxQ (maxq.tigris.org)" + EOL);
		result.append("// imports" + EOL);
		result.append("import junit.textui.TestRunner" + END_STATEMENT);
		result.append("import com.bitmechanic.maxq.HttpTestCase" + END_STATEMENT);
		result.append("import java.io.*" + END_STATEMENT);
		result.append("import java.util.*" + END_STATEMENT);
		result.append("import org.apache.commons.httpclient.NameValuePair" + END_STATEMENT);
		result.append("import org.apache.commons.httpclient.HttpStatus" + END_STATEMENT);

		if (jtidyConfigFile != null) {
			result.append(generateTidyImport());
		}

		if (isCpRspToStdout()) {
			result.append("import java.security.MessageDigest" + END_STATEMENT);
		}

		result.append(EOL + "// definition of test class" + EOL);
		result.append("public class " + getTestName() + " extends HttpTestCase {" + EOL);

		result.append("    public " + getTestName() + "(String name) {" + EOL);
		result.append("        super(name)" + END_STATEMENT);
		result.append("    }" + EOL + EOL);
		result.append("    public void setUp() {" + EOL).append("        //add setup code here if needed " + EOL).append("    }" + EOL + EOL);
		result.append("    public void tearDown() {" + EOL).append("        //add teardown code here if needed " + EOL).append("    }" + EOL + EOL);

		/*
		 * print the message digest routine, this is used as a checksum
		 * for people want to make sure they get the correct html output.
		 */
		if (isCpRspToStdout()) {
			result.append("    protected String getDigestStr(String httpString) throws Exception {").append(EOL);
			result.append("        MessageDigest md = MessageDigest.getInstance(\"SHA\")").append(END_STATEMENT);
			result.append("        md.update(httpString.getBytes())").append(END_STATEMENT);
			result.append("        byte[] digest = md.digest()").append(END_STATEMENT);
			result.append("        StringBuffer hexString = new StringBuffer()").append(END_STATEMENT);
			result.append("        for (int i = 0; i < digest.length; i++) {").append(EOL);
			result.append("            byte b = digest[i]").append(END_STATEMENT);
			result.append("            hexString.append(Integer.toHexString(0xFF & b))").append(END_STATEMENT);
			result.append("            hexString.append(\" \")").append(END_STATEMENT);
			result.append("        }").append(EOL).append(EOL);

			result.append("        return hexString.toString()").append(END_STATEMENT).append(EOL);
			result.append("    }").append(EOL).append(EOL);
		}

		result.append("    // Test method generated from the MaxQ Java generator").append(EOL);
		result.append("    public void test" + getTestName() + "() throws Exception {").append(EOL);
		result.append("        List list = new ArrayList()").append(END_STATEMENT);

		result.append("        String rsp").append(END_STATEMENT);
		result.append("        String filename").append(END_STATEMENT);
		result.append("        File htmlFile ").append(END_STATEMENT);
		result.append(EOL);
		result.append("        //^^^ Insert new recordings here.  (Do not remove this line.) ").append(END_STATEMENT);
		result.append("    }"); //close the method.
		result.append(EOL);
		result.append("}");
		log.debug("JTIDY:" + jtidyConfigFile);
		if (jtidyConfigFile != null) {
			result.append(setTidyConfig());
		}
		getScriptAdapter().append(result.toString());
	}

	private String generateTidyImport() {
		return ("import org.w3c.tidy.*" + END_STATEMENT);
	}

	private String setTidyConfig() {
		StringBuffer result = new StringBuffer();
		result.append(createStatement("//setup Tidy configurations"));
		result.append(createStatement("Tidy tidy = new Tidy()"));
		result.append(createStatement("tidy.setConfigurationFromFile(\"" + jtidyConfigFile + "\")"));
		return result.toString();
	}

	public void doCallUrl(String url, String method, String data, String contentLength) throws UserException {
		StringBuffer buffer = new StringBuffer();
		buffer.append(method).append("(\"").append(url).append("\"").append(data).append(contentLength).append(", list.toArray())");
		insert(createStatement(buffer.toString()));
	}

	public void doAssertResponse(String respCode) throws UserException {
		StringBuffer result = new StringBuffer();
		result.append(createStatement("System.out.println(\"Response code: \" + getResponseCode())"));

    String httpStatus = null;
    if(respCode != null && respCode.length() > 0) 
      httpStatus = HttpStatusMap.getHttpStatus(Integer.parseInt(respCode));
 
		if(httpStatus == null) 
      httpStatus = respCode;
		
    StringBuffer buffer = new StringBuffer();
		buffer.append("assertEquals(\"Assert number ").append(++assertNumber).append(" failed\", ").append(httpStatus).append(", getResponseCode())");
		result.append(createStatement(buffer.toString()));

		insert(result.toString());
	}

	public void doParameterList(Param[] params) throws UserException {
		insert(createStatement("list = new ArrayList()"));

		for (int i = 0; i < params.length; i++) {
			StringBuffer buffer = new StringBuffer();
			buffer.append("list.add(" + "new NameValuePair(\"").append(params[i].name).append("\", \"").append(params[i].value).append("\"))");
			insert(createStatement(buffer.toString()));
		}
	}

	public void doTestUrlMessage(String url) throws UserException {
		insert(createStatement("System.out.println(\"Testing URL: \" +replaceURL(\"" + url + "\"))"));
	}

	public void doSetData(String data) throws UserException {
		insert(createStatement("data = '" + data + "'"));
	}

	public void doTidyCode(String url) throws UserException {
		StringBuffer buf = new StringBuffer();
		if (jtidyConfigFile != null) {
			buf.append(createStatement("//validate html text via Tidy"));
			buf.append(createStatement("tidy.parse(getResponse().getInputStream(), System.out)"));
			buf.append(createStatement("if (tidy.getParseWarnings() + tidy.getParseErrors() > 0) {"));
			buf
					.append(createStatement("    System.out.println(\"<<Tidy report>> URL=" + url.trim()
							+ " WARNING=\"+tidy.getParseWarnings()+\", and ERROR=\"+tidy.getParseErrors())"));
			buf.append(createStatement("    System.out.println(\"+------------------------------------------------------------------------------+\")"));
			buf.append(createStatement("}"));
		}
		insert(buf.toString());
	}

	public void doResponseForStdOut(String url) throws UserException {
		StringBuffer result = new StringBuffer();
		result.append(createStatement("System.out.println(\"+--------Begin HTTP response for :" + url + "--------+\")"));
		result.append(createStatement("rsp = new String(getResponse().getData())"));
		result.append(createStatement("System.out.println(rsp)"));
		result.append(createStatement("System.out.println(\"+--------End HTTP response for :" + url + "--------+\")"));
		result.append(createStatement("System.out.println(\"+--------MessageDigest :\"+getDigestStr(rsp)+\"+--------+\")"));
		insert(result.toString());
	}

	public void doResponseForFile() throws UserException {
		StringBuffer result = new StringBuffer();
		// Save response in the tmp directory
		// Notice we are using HTTPResponse hashCode for filename
		result.append(createStatement("rsp = new String(getResponse().getData())"));
		result.append(createStatement("filename = getResponse().hashCode() + \".html\""));
		result.append(createStatement("System.out.println(\"+--------Save html output to :\"+filename+\"--------+\")"));
		result.append(createStatement("htmlFile = EditorPane.saveHTML(filename, rsp, \"tmp\")"));

		insert(result.toString());
	}

	public void doSetCharset(String cs) throws UserException {
		insert(createStatement("setCharset(\"" + cs + "\""));
	}

	public void doStartRecording() {
	}

	public void doStopRecording() {
	}

	public void doSave(String path, String fileName) {
		super.doSave(path, fileName);
		updateTestPath();
	}

	private void updateTestPath() {
		String pkg = getPackage();
		if (getScript() != null) {
			int startpos = getScript().indexOf("package ");
			if (startpos > -1) {
				startpos += 8;
				int end = getScript().indexOf(";", startpos);
				getScriptAdapter().replace(getPackage(), startpos, end);
			}
			else {
				if (!pkg.equals("")) {
					getScriptAdapter().insert("package " + pkg + ";" + EOL + EOL, 0);
				}
			}
		}
	}

	public void doLoad() {
		super.doLoad();
		String testPath = parseTestPath();
		log.debug("TestPath = " + testPath);
		setTestPath(parseTestPath());
	}

	public String parseTestName() {
		String testName = "";
		if (getScript() != null) {
			int startpos = getScript().indexOf("public class ");
			if (startpos > -1) {
				startpos += 13;
				int end = getScript().indexOf(" extends", startpos);
				testName = getScript().substring(startpos, end);
			}
		}
		return testName;
	}

	public void doHeaders(HeaderValue[] headers) {
	}

	private String parseTestPath() {
		String testPath = null;
		int pos = getPackageNameStartPos();
		if (pos > -1) {
			int end = getPackageNameEndPos();
			testPath = packageToPath(getScript().substring(pos, end).trim());
		}
		return testPath;
	}

	private int getPackageNameEndPos() {
		int pos = -1;
		if (getScript() != null) {
			pos = getScript().indexOf(";", getPackageNameStartPos());
		}
		return pos;
	}

	private int getPackageNameStartPos() {
		int pos = -1;
		if (getScript() != null) {
			pos = getScript().indexOf("package ");
			if (pos > -1) {
				pos += 7;
			}
		}
		return pos;
	}

	private String packageToPath(String s) {
		return getTestPath() + "\\" + s.replace('.', '\\');
	}

	private String getPackage() {
		String path = "";

		log.debug("baseclasspath = " + this.baseClasspath);
		log.debug("testpath = " + getTestPath());

		if (getTestPath() != null) {
			if (getTestPath().toLowerCase().startsWith(this.baseClasspath.toLowerCase()) && getTestPath().length() != baseClasspath.length()) {
				path = getTestPath().substring(this.baseClasspath.length() + 1);
			}
		}
		return path.replace('\\', '.');
	}

	private String getPackageDeclaration() {
		String pkgDeclaration = "";
		if (!getPackage().equals(""))
			pkgDeclaration = "package " + getPackage() + END_STATEMENT;

		return pkgDeclaration;
	}

	/**
	 * Returns the supplied string left padded
	 *
	 * @param text
	 */
	protected String createStatement(String text) {
		StringBuffer buffer = new StringBuffer();
		buffer.append("        ").append(text).append(END_STATEMENT);
		return buffer.toString();
	}

	public void doEndTransaction() {
		// Add a newline for readability.
		getScriptAdapter().append(EOL);
	}

	public void doQueryStringParameterList(Param[] params) throws UserException {
	}

	public void doBodyParameterList(Param[] params) throws UserException {
	}
}
