package com.bitmechanic.maxq.generator;

import org.python.core.PyString;

import com.bitmechanic.maxq.*;
import com.bitmechanic.maxq.Utils.UserException;

/**
 * Generates a Jython test case.
 * Take a look at the IScriptGenerator Interface to see how to specify which one you want.
 * The important information is that this class generates Jython code.
 *
 * @author Robert Dawson robert@rojotek.com
 * @author Wayne Hearn wayne_hearn@dell.com
 * @see IScriptGenerator
 */
public class JythonCodeGenerator extends AbstractCodeGenerator
{
  private boolean paramsExist = false;
  
  private boolean struts = false;

  public JythonCodeGenerator(IScriptAdapter adapter)
  {
    super(adapter, new String[]{"^class\\s+(\\w+)",
              "test = (\\w+)\\(\"\\w+\"\\)",
              "test = \\w+\\(\"(\\w+)\"\\)"});
  }

  public static String getGeneratorDescription()
  {
    return "Standard script";
  }
  
  public void setStruts(boolean struts){
    this.struts = struts;
  }

  public void doNew()
  {
    String s =
    "# Generated by MaxQ [" + getClass().getName() + "]" + EOL +
        "from PyHttpTestCase import PyHttpTestCase" + EOL +
        "from com.bitmechanic.maxq import Config" + EOL +
        "global validatorPkg" + EOL +
        "if __name__ == 'main':" + EOL +
        "    validatorPkg = Config.getValidatorPkgName()" + EOL +
        "# Determine the validator for this testcase." + EOL +
        "exec 'from '+validatorPkg+' import Validator'" + EOL +
        EOL;

    if (jtidyConfigFile != null) {
      s = s + "from org.w3c.tidy import *" + EOL;
    }

    if (isCpRspToStdout()) {
      s = s + "from java.security import MessageDigest" + EOL;
    }

    s = s +
        EOL +
        "# definition of test class" + EOL +
        "class " + getTestName() + "(PyHttpTestCase):" + EOL;

    /*
         * print the message digest routine, this is used as a checksum
         * for people want to make sure they get the correct html output.
         */
    if (isCpRspToStdout()) {
      s = s +
          "    def getDigestStr(self, http_str):" + EOL +
          "        md = MessageDigest.getInstance(\"SHA\")" + EOL +
          "        md.update(http_str.getBytes())" + EOL +
          "        digest = md.digest()" + EOL +
          "        hexString = StringBuffer()" + EOL +
          "        for i in digest:" + EOL +
          "            hexString.append(Integer.toHexString(0xFF & i))" + EOL +
          "            hexString.append(\" \")" + EOL +
          "        return hexString.toString()" + EOL +
          EOL;
    }

    if (jtidyConfigFile != null) {
      s = s +
          "    # Setup Tidy configurations" + EOL +
          "    tidy = Tidy()" + EOL +
          "    tidy.setConfigurationFromFile(r\"" + jtidyConfigFile + "\")" + EOL;
    }

    s = s +
        "    def runTest(self):" + EOL +
        "        self.msg('Test started')" + EOL +
        EOL +
        "    # ^^^ Insert new recordings here.  (Do not remove this line.)" + EOL;

    s = s +
        EOL +
        EOL +
        "# Code to load and run the test" + EOL +
        "if __name__ == 'main':" + EOL +
        "    test = " + getTestName() + "(\"" + getTestName() + "\")" + EOL;

    if (HttpTestCase.getUrlSearch() != null) {
      String replace = HttpTestCase.getUrlReplace();
      replace = quote(replace != null ? replace : "");
      String search = quote(HttpTestCase.getUrlSearch());
      s = s + "    test.setURLReplace(" + replace + ", " + search + ")" + EOL;
    }

    s = s + "    test.Run()";

    getScriptAdapter().append(s);
  }

  public void doParameterList(Param[] params) throws Utils.UserException
  {
    insertStmt("params = [");
    int i;
    for (i = 0; i < params.length; i++) {
      String s;
      if (params[i].name.toLowerCase().equals("org.apache.struts.taglib.html.TOKEN".toLowerCase())){
        s = "    (" + quote(params[i].name) + ", "
                    +  "strutsToken" + "),";
        } else{
          s = "    (" + quote(params[i].name) + ", "
                      + quote(params[i].value) + "),";
        }
      if (i == params.length - 1)
        s = s + "]";
      insertStmt(s);
    }
    paramsExist = true;
  }

  public void doCallUrl(String url, String method, String data, String contentLength) throws Utils.UserException
  {
    if (this.struts) {
      insertStmt("strutsToken=self.getStrutsToken()");
      setStruts(false);
    }
    insertStmt("url = \"" + url + "\"");
    if (!paramsExist) {
      insertStmt("params = None");
    }
    insertStmt("Validator.validateRequest(self, self.getMethod(), \"" + method + "\", url, params)");
    insertStmt("self." + method + "(" +
            "url" +
            data +
            contentLength +
            ", params)");
    paramsExist = false;
  }

  public void doSetData(String data) throws Utils.UserException
  {
    insertStmt("data = '" + data + "'");
  }

  public void doAssertResponse(String respCode) throws Utils.UserException
  {
    insertStmt("self.msg(\"Response code: %s\" % self.getResponseCode())");
    insertStmt("self.assertEquals(\"Assert number " + ++assertNumber + " failed\", " + respCode +
            ", self.getResponseCode())");
    insertStmt("Validator.validateResponse(self, self.getMethod(), url, params)");
  }

  public void doTidyCode(String url) throws Utils.UserException
  {
    if (jtidyConfigFile != null) {
      insertStmt("# Validate html text via Tidy");
      insertStmt("tidy.parse(self.getMethod().getInputStream(), System.out)");
      insertStmt("if tidy.parseWarnings + tidy.parseErrors > 0:");
      insertStmt("  self.msg('''<<Tidy report>> URL=" + url.trim() + " WARNING=%d, and ERROR=%d\n''' % (tidy.parseWarnings, tidy.parseErrors))");
      insertStmt("  self.msg('') '''+------------------------------------------------------------------------------+'''");
    }
  }

  public void doResponseForStdOut(String url) throws Utils.UserException
  {
    insertStmt("self.msg('''+--------Begin HTTP response for :" + url + "--------+''')");
    insertStmt("rsp = String(self.getMethod().getData())");
    insertStmt("self.msg(rsp)");
    insertStmt("self.msg('''+--------End HTTP response for :" + url + "--------+''')");
    insertStmt("self.msg('''+--------MessageDigest :%s--------+''' % self.getDigestStr(rsp))");
  }

  public void doResponseForFile() throws Utils.UserException
  {
    // Save response in the tmp directory
    // Notice we are using HTTPResponse hashCode for filename
    insertStmt("rsp = String(self.getMethod().getData())");
    insertStmt("filename = self.getMethod().hashCode().toString() + '.html'");
    insertStmt("self.msg('''+--------Save html output to :%s--------+''' % filename)");
    insertStmt("htmlFile = EditorPane.saveHTML(filename, rsp, 'tmp')");
  }

  public void doTestUrlMessage(String url) throws Utils.UserException
  {
    insertStmt("self.msg(\"Testing URL: %s\" % self.replaceURL('''" + HttpTestCase.replaceURL(url) + "'''))");
  }

  public String[] getValidFileExtensions()
  {
    return new String[]{".py"};
  }

  public void doStartRecording()
  {
  }

  public void doStopRecording()
  {
  }

  /**
   * Appends an EOL char to scriptAdapter
   */
  public void doEndTransaction() throws Utils.UserException
  {
    // Add a newline for readability.
    insertStmt("");
  }

  public void doSetCharset(String cs) throws Utils.UserException
  {
    insert("   self.setCharset("+quote(cs)+")");
  }

  public void doHeaders(HeaderValue[] headers)
  {
    //To change body of implemented methods use File | Settings | File Templates.
  }

  /**
   * Inserts a string into the script, indented and with a newline appended.
   */
  private void insertStmt(String s) throws Utils.UserException
  {
    insert("        " + s + EOL);
  }

  /**
   * Quotes the string using Python conventions.
   */
  private static String quote(String str)
  {
    return new PyString(str).__repr__().toString();
  }

  public void doQueryStringParameterList(Param[] params) throws UserException {
  }

  public void doBodyParameterList(Param[] params) throws UserException {
  }
}
