package com.bitmechanic.maxq;

import java.util.Vector;

/**
 * Class that represents a single request that has been recorded by the
 * ProxyRecorder.
 */
public class Action
{
  private static String LINE_SEP = System.getProperty("line.separator");

  private String url, method, expected_result;
  private int delayTime, expected_size, timeout;
  private Vector headers;
  private Vector queryStringParams;
  private Vector bodyParams;
  private String multiPartData;

  public Action()
  {
    headers = new Vector();
    queryStringParams = new Vector();
    bodyParams = new Vector();
    expected_result = "";
    multiPartData = "";
  }

  public void setUrl(String url)
  {
    this.url = url;
  }

  public String getUrl()
  {
    return url;
  }

  public void setMethod(String method)
  {
    this.method = method;
  }

  public String getMethod()
  {
    return method;
  }

  public void setExpectedResult(String str)
  {
    expected_result = str;
  }

  public String getExpectedResult()
  {
    return expected_result;
  }

  public void setDelayTime(int seconds)
  {
    delayTime = seconds;
  }

  public int getDelayTime()
  {
    return delayTime;
  }

  public void setExpectedSize(int bytes)
  {
    expected_size = bytes;
  }

  public int getExpectedSize()
  {
    return expected_size;
  }

  public void setTimeout(int seconds)
  {
    timeout = seconds;
  }

  public int getTimeout()
  {
    return timeout;
  }


  public int getHeaderCount()
  {
    return headers.size();
  }

  public HeaderValue[] getHeaders()
  {
    HeaderValue list[] = new HeaderValue[headers.size()];
    headers.copyInto(list);
    return list;
  }

  public void addHeader(HeaderValue p)
  {
    headers.addElement(p);
  }

  public void removeHeader(HeaderValue p)
  {
    headers.removeElement(p);
  }  

  public int getParamsCount()
  {
    return getParams().length;
  }

  public int getQueryStringParamCount()
  {
    return queryStringParams.size();
  }

  public int getBodyParamCount()
  {
    return bodyParams.size();
  }

  public Param[] getQueryStringParams()
  {
    Param list[] = new Param[queryStringParams.size()];
    queryStringParams.copyInto(list);
    return list;
  }

  public Param[] getBodyParams()
  {
    Param list[] = new Param[bodyParams.size()];
    bodyParams.copyInto(list);
    return list;
  }
  
  /**
   * Return query string and body parameter merged.
   */
  public Param[] getParams()
  {
    Vector v = (Vector)queryStringParams.clone();
    v.addAll(bodyParams);
    Param list[] = new Param[v.size()];
    v.copyInto(list);
    return list;
  }

  public void addQueryStringParam(Param p)
  {
    queryStringParams.addElement(p);
  }

  public void addBodyParam(Param p)
  {
    bodyParams.addElement(p);
  }

  public void removeQueryStringParam(Param p)
  {
	queryStringParams.removeElement(p);
  }

  public void removeBodyParam(Param p)
  {
	bodyParams.removeElement(p);
  }


  public String toXML()
  {
    StringBuffer str = new StringBuffer(1024);
    str.append("    ");
    str.append("<action url=\"").append(url).append("\" ");
    str.append(LINE_SEP).append("            ");
    str.append("method=\"").append(method).append("\" ");
    str.append(LINE_SEP).append("            ");
    str.append("delayTime=\"").append(delayTime).append("\" ");
    str.append(LINE_SEP).append("            ");
    str.append("expected-result=\"").append(expected_result).append("\"");
    str.append(LINE_SEP).append("            ");
    str.append("expected-result-size=\"").append(expected_size).append("\"");
    str.append(LINE_SEP).append("            ");
    str.append("timeout=\"").append(timeout).append("\">");
    Param params[] = getQueryStringParams();
    for (int i = 0; i < params.length; i++) {
      str.append(LINE_SEP).append("                ");
      str.append("<param name=\"").append(params[i].name);
      str.append("\" value=\"").append(params[i].value).append("\"/>");
    }
    params = getBodyParams();
    for (int i = 0; i < params.length; i++) {
      str.append(LINE_SEP).append("                ");
      str.append("<param name=\"").append(params[i].name);
      str.append("\" value=\"").append(params[i].value).append("\"/>");
    }
    str.append(LINE_SEP);
    str.append("    </action>");
    return str.toString();
  }

  public void setMultiPartData(String multiPartData)
  {
    this.multiPartData = multiPartData;
  }

  public String getMultiPartData()
  {
    return multiPartData;
  }
}
