package com.bitmechanic.maxq;

import junit.framework.TestCase;
import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.MultipartPostMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.multipart.StringPart;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.python.core.PyDictionary;
import org.python.core.PyString;
import org.python.core.PyTuple;

import javax.swing.*;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;

public class HttpTestCase extends TestCase
{
  private static final Log log = LogFactory.getLog(HttpTestCase.class);
  private Config config = Config.getConfig();
  //    private String host = "127.0.0.1";
  //    private int port = 80;

  private HashMap connections = new HashMap();

  protected HttpMethod method = null;

  protected HttpClient client = new HttpClient();

  protected String data;

  private boolean followRedirects = false;

  private static String urlSearch, urlReplace;

  private String charset;
  
  public void setCharset(String cs) {
    charset = cs;
  }
  
  public static String getUrlReplace()
  {
    return urlReplace;
  }

  public static String getUrlSearch()
  {
    return urlSearch;
  }

  public static void setURLReplace(String search, String replace)
  {
    // Let this method be called only once. If specified through the
    // command-line options, then script can't change it, so essentially the
    // call from the script works like a default.
    if (urlSearch == null) {
      urlSearch = search.trim();
      urlReplace = replace.trim();
    }
    log.debug("Ignoring setURLReplace call: search: " + search + " replace: " + replace);
  }

  public static String replaceURL(String url)
  {
    if (urlSearch == null || urlReplace == null) {
      return url;
    } else {
      int pos = url.indexOf(urlSearch);
      if (pos == -1) {
        return url;
      } else {
        String str = url.substring(0, pos) + urlReplace +
            url.substring(pos + urlSearch.length());
        return str;
      }
    }
  }


  public HttpTestCase(String name)
  {
    super(name);
    Config.ProxySettings proxy = config.getProxySettings();
    if (proxy != null) {
      HostConfiguration hc = client.getHostConfiguration();
      hc.setProxy(proxy.host, proxy.port);
    }
  }

  /**
   * This is called when our test scripts are run standalone, not when we are
   * running a JUnit test harness.  Therefore we need not involve JUnit
   * as it just confuses the error output.
   */
  public void Run() throws Throwable
  {
    setUp();
    runTest();
    tearDown();
  }

  public HttpMethod getMethod()
  {
    return method;
  }

  private void cleanupMethod()
  {
    if (method != null) {
      method.releaseConnection();
      method.recycle();
      method = null;
    }
  }

  public void get(String url) throws IOException
  {
    get(url, null);
  }

  public void get(String url, Object[] args) throws IOException
  {
    cleanupMethod();

    url = replaceURL(url);
    method = new GetMethod(url);

    if (args != null) {
      method.setQueryString(paramsToNV(args));
    }

    method.setFollowRedirects(followRedirects);
    client.executeMethod(method);
  }

  public void post(String url) throws IOException
  {
    post(url, null);
  }

  public void post(String url, Object[] args) throws IOException
  {
    cleanupMethod();
    url = replaceURL(url);
    PostMethod post = new PostMethod(url); // so we don't have to cast

    if(charset != null) {
      post.setRequestHeader("Content-Type",
                            PostMethod.FORM_URL_ENCODED_CONTENT_TYPE
                            + "; charset=" + charset );
    }
    
    if (args != null) {
      post.setRequestBody(paramsToNV(args));
    }

    method = post;
    method.setFollowRedirects(followRedirects);
    client.executeMethod(method);
  }

  /**
   * Performs a POST in the form of multipart/form-data
   */
  public void postMultiPart(String url, String data, int contLen) throws IOException
  {
    postMultiPart(url, data, contLen, null);
  }

  /**
   * Performs a POST in the form of multipart/form-data
   */
  public void postMultiPart(String url, String data, int contLen, Object[] args) throws IOException
  {
    cleanupMethod();
    url = replaceURL(url);

    MultipartPostMethod multipartPost = new MultipartPostMethod(url);
    multipartPost.addPart(new StringPart("data", data));
    if (args != null) {
      multipartPost.setQueryString(paramsToNV(args));
    }

    method = multipartPost;
    method.setFollowRedirects(followRedirects);
    client.executeMethod(method);
  }

  public void setConnections(HttpTestCase test)
  {
    test.connections = this.connections;
  }

  /**
   * Converts parameters in naturally python representation (a list of
   * tuples) to the form expected by HttpClient.
   */
  public NameValuePair[] paramsToNV(Object[] params)
  {
	int i;
    NameValuePair[] res = new NameValuePair[params.length];
    for (i = 0; i < params.length; i++) {
      Object param = params[i];
      if(param instanceof PyTuple) {
	      PyTuple pyParam = (PyTuple) param;
	      res[i] = new NameValuePair(pyParam.__getitem__(0).toString(),
	    		  pyParam.__getitem__(1).toString());
      }
      else if(param instanceof NameValuePair) 
    	  res[i] = (NameValuePair)param;
    }
    return res;
  }

  /**
   * URL Decodes the given string.  GET/POST data recored by the recorder
   * that have + or % in the value get passed through in the generated
   * python code.
   * <p/>
   * Code reproduced from a comp.lang.java.programmer USENET post.
   */
  public String urlDecode(String s)
  {
    return staticUrlDecode(s);
  }

  public static String staticUrlDecode(String s)
  {
    ByteArrayOutputStream out = new ByteArrayOutputStream(s.length());
    for (int count = 0; count < s.length(); count++) {
      if (s.charAt(count) == '%') {
        count++;
        if(count < s.length()) {
          int a = Character.digit(s.charAt(count++), 16);
          a = a << 4;
          int b = Character.digit(s.charAt(count), 16);
          // prefix ' (ascii 39) or " (ascii 132) with '\' (ascii 92)
          if (a + b == 39 || a + b == 132) out.write(92);
          out.write(a + b);
        }
        else {
          // compensate for URLs that end in %
          out.write('%');
        }
      } else {
        if (s.charAt(count) == '+')
          out.write(' ');
        else
          out.write(s.charAt(count));
      }
    }

    return (out.toString());
  }

  /**
   * Checks to see if the HTTP response is 200, 302, or 304.  If not, the
   * assertion fails.
   */
  protected void responseOK() throws URIException
  {
    int status = getResponseCode();
    assertTrue("Invalid HTTP response: " + method + " for URI: " +
            method.getURI(),
        (status == 200 || status == 302 || status == 304));
  }

  /**
   * Checks if the response contains the uri provided.  This is useful to
   * check if a page sent a redirect to the client.  If it sent a redirect,
   * will will point at the target path that the client was redirected to.
   *
   * @param uri URI to check the URI against
   * @return true if uri is in the URI of the response
   */
  protected boolean responseContainsURI(String uri) throws URIException
  {
    return (method != null &&
                 method.getURI().getPath().indexOf(uri) != -1);
  }

  /**
   * Checks if the text is in the response data.  This is useful if you want
   * to make surethat the server included a given String in the response.
   *
   * @param text String that the response should contain
   * @return True if text is in the body of the HTTP response
   */
  protected boolean responseContains(String text)
  {
    if (method == null || method.getResponseBody() == null) return false;

    data = method.getResponseBodyAsString();
    return data.indexOf(text) != -1;
  }

    protected void printResponse()
    {
	System.err.println(new String(method.getResponseBody()));
    }
    
    /**
     * Converts the byte[] method.response as a string, regarding to the encoding
     */
    public String getResponse() {
        return method.getResponseBodyAsString();
    }
    
    /**
     * Just returns the response as an array of bytes
     */
    public byte[] getResponseAsBytes() {
        return method.getResponseBody();
    }

  public int getResponseCode()
  {
    return getMethod().getStatusCode();
  }

  /**
   * Returns response headers as a Python dictionary.
   */
  public PyDictionary getResponseHeader()
  {
    PyDictionary dict = new PyDictionary();
    Header[] headers = method.getResponseHeaders();
    int i;
    for (i = 0; i < headers.length; i++)
      dict.__setitem__(new PyString(headers[i].getName()),
          new PyString(headers[i].getValue()));
    return dict;
  }

  /**
   * Returns any argument for the script.  This argument is normally provided
   * using the "--script-arg" command-line option.
   */
  public String getScriptArg()
  {
    return config.getScriptArg();
  }

  /**
   * Puts up an OK/Cancel message box and returns whether the user clicked OK.
   */
  public boolean userConfirm(String msg)
  {
    int rc = JOptionPane.showConfirmDialog(null, msg, "MaxQ",
        JOptionPane.OK_CANCEL_OPTION);
    return rc == JOptionPane.OK_OPTION;
  }

  /**
   * Puts up a message box with a prompt and returns whatever the user
   * enters.  If the user clicks Cancel then null is returned.
   */
  public String userInput(String msg)
  {
    return JOptionPane.showInputDialog(null, msg, "MaxQ",
        JOptionPane.QUESTION_MESSAGE);
  }
  
  /** This operation returns the value of the Struts TOKEN in the response
   * @return strutsToken A String with the value of the Struts TOKEN
   */
  public String getStrutsToken() {
    final String strutsTokenID = "org.apache.struts.taglib.html.TOKEN";
    final String valueString = "value=\"";
    final int strutsTokenLenght = 32; 
    	
   	String responseString = new String(method.getResponseBodyAsString());
   	//System.out.println(responseString);
   	String strutsToken = null;
   	
   	int tokenIDIndex = responseString.toLowerCase().indexOf(strutsTokenID.toLowerCase());
    int nextValue = responseString.toLowerCase().indexOf(valueString.toLowerCase(), tokenIDIndex);
    	
    strutsToken = responseString.substring(nextValue + valueString.length(),nextValue + valueString.length() + strutsTokenLenght);
    return strutsToken;
  }


}
