package com.bitmechanic.maxq;

import org.python.core.Py;
import org.python.core.PyException;
import org.python.util.PythonInterpreter;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class RunTestDialog extends JDialog
    implements ActionListener, Runnable
{
  private Thread runner;

  private String testScript;

  private JTextArea textArea;
  private JPanel buttonPane;
  private JButton okButton = new JButton("OK");
  private JButton cancelButton = new JButton("Cancel");

  public RunTestDialog(JFrame frame, String testScript)
  {
    super(frame, true);


    setTitle("Running Test...");
    Dimension size = new Dimension(600, 400);
    setSize(size);

    // prevent user from using close box to exit dialog
    addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent we)
      {
        // no-op
      }
    });

    textArea = new JTextArea();
    textArea.setFont(new Font("Courier", Font.PLAIN, 12));
    textArea.setLineWrap(false);
    JScrollPane scrollPane = new JScrollPane(textArea);

    buttonPane = new JPanel();
    buttonPane.setLayout(new GridLayout(1, 2));
    buttonPane.add(cancelButton);

    Border border = BorderFactory.createCompoundBorder(BorderFactory.createRaisedBevelBorder(), BorderFactory.createLoweredBevelBorder());
    buttonPane.setBorder(border);

    cancelButton.addActionListener(this);
    okButton.addActionListener(this);

    // begin running test
    this.testScript = testScript;
    runner = new Thread(this);
    runner.start();

    getContentPane().add(scrollPane, BorderLayout.CENTER);
    getContentPane().add(buttonPane, BorderLayout.SOUTH);
    setVisible(true);
  }

  /*
  private void parseLines(String script) {
      ArrayList arr = new ArrayList();
      StringTokenizer st = new StringTokenizer(script, "\n", true);
      while (st.hasMoreTokens()) {
          arr.add(st.nextToken());
      }
      lines = new String[arr.size()];
      arr.toArray(lines);
  }
  */

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();

    if (source == cancelButton) {
      runner.interrupt();
      try {
        runner.join();
      } catch (InterruptedException ex) {
        // Someone is not willing to wait.  No problem, it'll finish
        // up eventually.
      }
      dispose();
    } else if (source == okButton) {
      dispose();
    }
  }

  public void run()
  {
    PythonInterpreter python = new PythonInterpreter();
    DialogWriter writer = new DialogWriter();
    python.setErr(writer);
    python.setOut(writer);

    try {
      python.exec(testScript);
      setTitle("Test Run Completed");
      textArea.append("Test Ran Successfully");
    } catch (PyException e) {
      // Unlike e.printStackTrace(), this does not include the Java stack
      // so it is much more brief.
      Py.printException(e);
      setTitle("Test Run Failed");
    } // Other exceptions indicate a problem with MaxQ.

    // swap buttons
    buttonPane.removeAll();
    buttonPane.add(okButton);
    buttonPane.validate();
  }

  class DialogWriter extends java.io.Writer
  {
    public void close()
    {
    }

    public void flush()
    {
    }

    public void write(char[] cbuf, int off, int len)
    {
      textArea.append(new String(cbuf, off, len));
    }
  }

}
