package com.bitmechanic.maxq;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Created by IntelliJ IDEA.
 * User: whearn
 * Date: Aug 23, 2003
 * Time: 12:34:05 AM
 * To change this template use Options | File Templates.
 */
public class Utils
{
  public final static String SPACE = "                                                                                ";
  private static final Log log = LogFactory.getLog(Utils.class);

  /**
   * For exceptions whose messages have been composed so that they can be
   * understood by the user.  There is no need to show a stack trace for
   * these.
   */
  public static class UserException extends Exception
  {
    public UserException(String s)
    {
      super(s);
    }
  }

  /**
   * For silently aborting without showing any error.
   */
  public static class SilentException extends Exception
  {
    public SilentException()
    {
      super("silent abort");
    }
  }

  /**
   * Save HTML string to a file under the specified directory.
   * Target file is overwritten if already exists.
   *
   * @param htmlFileName HTML filename
   * @param str          HTML text
   * @param dir_str      directory under which the HTML file will be created
   * @return file representing the HTML file created
   */
  public static File saveHTML(String htmlFileName, String str, String dir_str)
  {

    if (str == null) {
      log.error("Argument is null: EditorPane.saveHTML()");
      return null;
    }

    File dir;
    if (dir_str == null)
      dir = new File("Tempfiles");
    else
      dir = new File(dir_str);

    dir.mkdir();

    File htmlFile;
    try {
      htmlFile = new File(dir, htmlFileName);
      if (htmlFile.exists()) {
        htmlFile.delete();
        log.warn("deleting duplicated file: " + htmlFileName);
      }
      htmlFile.createNewFile();
      PrintWriter writer = new PrintWriter(new FileWriter(htmlFile));
      writer.print(str);
      writer.close();
    } catch (Exception e) {
      e.printStackTrace();
      return null;
    }

    return htmlFile;
  }

  public static String[] splitString(String str, String delim)
  {
    StringTokenizer st = new StringTokenizer(str, delim);
    String[] res = new String[st.countTokens()];
    int i = 0;
    while (st.hasMoreTokens()) {
      res[i] = st.nextToken();
      i = i + 1;
    }
    return res;
  }

  public static String replace(String str, String search, String replace)
  {
    int pos = str.indexOf(search);
    if (pos == -1) return str;
    StringBuffer buff = new StringBuffer(str.length() + 32);
    int start = 0;
    while (pos != -1 && start < str.length()) {
      buff.append(str.substring(start, pos));
      buff.append(replace);

      start = pos + search.length();
      if (start < str.length()) pos = str.indexOf(search, start);
    }
    if (start < str.length()) buff.append(str.substring(start));

    return buff.toString();
  }

  public static List getParams(String query)
  {
    List paramList = new ArrayList();
    if (query != null) {
      query = query.trim();
      log.debug("  parsing: " + query);
      String items[] = Utils.splitString(query, "&");
      for (int i = 0; i < items.length; i++) {
        int pos = items[i].indexOf("=");
        if (pos != -1) {
          Param p = new Param();
          paramList.add(p);
          p.name = items[i].substring(0, pos);
          if (pos < (items[i].length() - 1))
            p.value = items[i].substring(pos + 1);
          else
            p.value = "";
        }
      }
    }
    return paramList;
  }
}
